/* Determines whether the light ray is to strike the face of the PMT, updates
the stats for later repositioning of the PMT.  See my notes for the details of
the geometry used here.	*/
#include "raytr.h"

#define RD 57.29577951

int hit_PMT(m,r,out,stat)
MIRROR *m;
RAY    *r;
FILE   *out;
STATS  *stat;
{
    double x,y,lx,ly,ll,ll_pr,s,phi,psi,zeta;

#ifdef DEBUG    
printf("ray:  x,y:  % f,% f   theta:  % f\n",r->x,r->y,r->theta*RD);
printf("mirror:  F: % f,% f,  phi  :  % f\n",m->Fx,m->Fy,m->phi*RD);
#endif

    lx = m->Fx - r->x;
    ly = m->Fy - r->y;
    ll = sqrt( lx*lx + ly*ly );
    psi = acos( sin(r->theta)*lx/ll + cos(r->theta)*ly/ll );
    phi = PI - fabs(m->phi - r->theta);

#ifdef DEBUG
printf("l       :   % f,% f, psi:     % f,  phi:  % f\n",lx,ly,psi*RD,phi*RD);
#endif

    s = ll*sin(psi)/sin(phi);

#ifdef DEBUG
printf("  s: % f\n",s);
scanf("%lf",&y);
#endif

    zeta = PI - phi - psi;
    ll_pr = sqrt( s*s + ll*ll - fabs(2.0*s*ll*cos(zeta)) );
    x = r->x + ll_pr*sin(r->theta);
    y = r->y + ll_pr*cos(r->theta);
    
    if( y < m->Fy )
	s *= -1.0;

    stat->angle = phi*57.296;
    stat->s = s;
    stat->sum += s;
    stat->sumsqr += s*s;
    if( stat->n == 0.0 )
	stat->min = stat->max = s;
    stat->n++;
    if( s < stat->min )
	stat->min = s;
    if( s > stat->max )
	stat->max = s;

    if( fabs(s) <= PMT_SIZE )
       	{
       	fprintf(out,"%f %f\n%f %f\njoin dots\n",(float)r->x,(float)r->y,(float)x,(float)y);
       	num_caught++;
       	return( 1 );
       	}
    else 
       	{
       	return( 0 );
       	}
}
